<!DOCTYPE style-sheet PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN" [
<!ENTITY missing-standard-procedures system "std-proc.dsl">
]>
<style-specification>

(declare-flow-object-class element
  "UNREGISTERED::James Clark//Flow Object Class::element")
(declare-flow-object-class empty-element
  "UNREGISTERED::James Clark//Flow Object Class::empty-element")
(declare-flow-object-class document-type
  "UNREGISTERED::James Clark//Flow Object Class::document-type")
(declare-flow-object-class entity
  "UNREGISTERED::James Clark//Flow Object Class::entity")

&missing-standard-procedures; 

; Suspiciously missing counterpart of node-list->list.
(define (list->node-list l)
  (apply node-list l))

(define (copy-attributes #!optional (nd (current-node)))
  (let loop ((atts (named-node-list-names (attributes nd))))
    (if (null? atts)
        '()
        (let* ((name (car atts))
               (value (attribute-string name nd)))
          (if value
              (cons (list name value)
                    (loop (cdr atts)))
              (loop (cdr atts)))))))

(define (string-ci>? s1 s2)
  (let ((len1 (string-length s1))
	(len2 (string-length s2)))
    (let loop ((i 0))
      (cond ((= i len1) #f)
	    ((= i len2) #t)
	    (#t (let ((c1 (index-char-val (string-ref s1 i)))
		      (c2 (index-char-val (string-ref s2 i))))
		  (cond
		   ((= c1 c2) (loop (+ i 1)))
		   (#t (> c1 c2)))))))))

(define (index-char-val ch)
  (case ch
    ((#\A #\a) 65)
    ((#\B #\b) 66)
    ((#\C #\c) 67)
    ((#\D #\d) 68)
    ((#\E #\e) 69)
    ((#\F #\f) 70)
    ((#\G #\g) 71)
    ((#\H #\h) 72)
    ((#\I #\i) 73)
    ((#\J #\j) 74)
    ((#\K #\k) 75)
    ((#\L #\l) 76)
    ((#\M #\m) 77)
    ((#\N #\n) 78)
    ((#\O #\o) 79)
    ((#\P #\p) 80)
    ((#\Q #\q) 81)
    ((#\R #\r) 82)
    ((#\S #\s) 83)
    ((#\T #\t) 84)
    ((#\U #\u) 85)
    ((#\V #\v) 86)
    ((#\W #\w) 87)
    ((#\X #\x) 88)
    ((#\Y #\y) 89)
    ((#\Z #\z) 90)

    ((#\ ) 32)

    ((#\0) 48)
    ((#\1) 49)
    ((#\2) 50)
    ((#\3) 51)
    ((#\4) 52)
    ((#\5) 53)
    ((#\6) 54)
    ((#\7) 55)
    ((#\8) 56)
    ((#\9) 57)

    (else 0)))

; Sort an alist. Each member of al should be a pair whose car 
; must be a string which is used as sort key.
(define (sort-alist al)
  (letrec ((list-head (lambda (l n)
			(if (> n 0)
			    (cons (car l) (list-head (cdr l) (- n 1)))
			    '())))
	   (merge (lambda (al1 al2)
		    (cond ((null? al1) al2)
			  ((null? al2) al1)
			  ((string-ci>? (car (car al2)) (car (car al1)))
			   (cons (car al2) (merge al1 (cdr al2))))
			  (#t (cons (car al1) (merge (cdr al1) al2)))))))
    (let* ((ll (length al))
	   (ldiv2 (quotient ll 2)))
      (if (> 2 ll)
	  al
	  (merge (sort-alist (list-head al ldiv2))
		 (sort-alist (list-tail al ldiv2)))))))

; Turn a node list into an alist with the help of proc which must 
; be a function turning a singleton node list into a pair.
(define (node-list->alist proc nl)
  (map proc (node-list->list nl)))

; Turn an alist into a node list.
(define (alist->node-list al)
  (list->node-list (map cdr al))) 

; Return a node list containing all nodes whose id's occur as the value 
; of a bibref element in the subtree below the current node. The node 
; list is sorted alphabetically by author.
(define (references) 
  (alist->node-list 
   (sort-alist 
    (node-list->alist 
     (lambda (snl) (cons (data (select-elements (descendants snl) "AUTHOR")) snl)) 
     (node-list-remove-duplicates
      (node-list-map 
       (lambda (snl) (element-with-id (attribute-string "CITE" snl)))
       (select-elements (descendants (current-node)) "BIBREF")))))))

; parameters

(define instance-sysid "bibdoc.sgm2")
(define dtd-sysid "bib.dtd")

; construction rules

(root 
 (case (gi (node-property 'docelem (current-node))) 
   (("DOC1") (make entity
		   system-id: instance-sysid
		   (make document-type
			 name:  "DOC2" 
			 system-id: dtd-sysid)
		   (process-children)))
   (("DOC2") (error "I can only work on documents of type DOC1."))))
 
(default
  (cond
   ((node-list-empty? (node-property 'content (current-node)))
    (make empty-element
	  attributes: (copy-attributes)))
   (else
    (make element
	  attributes: (copy-attributes)))))

(element BIBLIOGRAPHY
  (empty-sosofo))

(element DOC1
  (make element
	gi: "DOC2"
	attributes: (copy-attributes)
	(process-children)
	(make element
	      gi: "REFERENCES"
	      (process-node-list (references)))))
 
</style-specification>
